"""Acquia Cloud API server resource. """

from acapi.resources.acquiaresource import AcquiaResource


class Server(AcquiaResource):
    """Acquia Cloud API Server."""

    valid_keys = ['name', 'fqdn', 'services', 'status', 'ami_type',
                  'ec2_region', 'ec2_availability_zone']



""" Imports all the resources. """

# Base classes
from acapi.resources.acquiadata import AcquiaData
from acapi.resources.acquialist import AcquiaList
from acapi.resources.acquiaresource import AcquiaResource

# Databases can be backed up
from acapi.resources.backup import Backup
from acapi.resources.backuplist import BackupList

# Environments have databases
from acapi.resources.database import Database
from acapi.resources.databaselist import DatabaseList

# Environments have domains
from acapi.resources.domain import Domain
from acapi.resources.domainlist import DomainList

# Sites reference environments
from acapi.resources.environment import Environment
from acapi.resources.environmentlist import EnvironmentList

# Sites have servers
from acapi.resources.server import Server
from acapi.resources.serverlist import ServerList

# Sites have tasks
from acapi.resources.task import Task
from acapi.resources.tasklist import TaskList

# Site is our top level
from acapi.resources.site import Site
from acapi.resources.sitelist import SiteList

# Users are top level
from acapi.resources.user import User

__all__ = [
    'AcquiaData',
    'AcquiaList',
    'AcquiaResource',
    'Backup',
    'BackupList',
    'Database',
    'DatabaseList',
    'Domain',
    'DomainList',
    'Environment',
    'EnvironmentList',
    'Server',
    'ServerList',
    'Task',
    'TaskList',
    'Site',
    'SiteList',
    'User',
]



"""Acquia Cloud API user resource."""

from acapi.resources.acquiaresource import AcquiaResource


class User(AcquiaResource):
    """Acquia Cloud API user resource."""

    #: Valid keys for a user.
    valid_keys = ['authenticated_as']

    def drushrc(self):
        """Fetch all the drush aliases for sites the user has access to.

        The json+PHP output of this isn't very useful in python.

        Returns
        -------
        dict
            Collection of PHP code snippets.
        """
        uri = '{}/drushrc'.format(self.uri)
        response = self.request(uri=uri)
        return response



"""Acquia Cloud API Exceptions."""

from pprint import pformat


class AcquiaCloudException(Exception):
    """Generic Acquia Cloud API Exception.

    All ACAPI exceptions should extend this class.
    """

    pass


class AcquiaCloudNoDataException(AcquiaCloudException):
    """No data found exception."""

    pass


class AcquiaCloudTaskFailedException(AcquiaCloudException):
    """An Acquia task failure exception."""

    def __init__(self, message, task):
        """Constructor.

        Parameters
        ----------
        message: str
            The error message.
        task: Task
            The Task object for the task that failed.
        """
        super(AcquiaCloudTaskFailedException, self).__init__(message)
        self.message = message
        self.task = task

    def __str__(self):
        """Return the string representation of the exception.

        Returns
        -------
        str
            The error message and pretty printed Task object properties.
        """
        task = pformat(self.task, indent=4)
        return "{msg}\n{task}".format(msg=self.message, task=task)
